@@ -14,7 +14,9 @@
 # classes.py
 # Copyright (C) 2014-2015 Fracpete (pythonwekawrapper at gmail dot com)
 
+import os
 import inspect
+import argparse
 import json
 import logging
 import re
@@ -23,6 +25,10 @@ import javabridge
 from javabridge import JWrapper, JClassWrapper
 from javabridge.jutil import JavaException
 from weka.core import types as arrays
+import weka.core.jvm as jvm
+
+# logging setup
+logger = logging.getLogger("weka.core.classes")
 
 
 def get_class(classname):
@@ -1171,7 +1177,7 @@ class Tags(JavaObject):
         :param tags: the list of Tag objects to use
         :type tags: list
         """
-        if not tags is None:
+        if tags is not None:
             jarray = JavaArray(JavaArray.new_instance("weka.core.Tag", len(tags)))
             for i in range(len(tags)):
                 jarray[i] = tags[i]
@@ -1196,7 +1202,7 @@ class Tags(JavaObject):
         :return: the tag
         :rtype: Tag
         """
-        return Tag(self.array[item])
+        return Tag(self.array[item].jobject)
 
     def __setitem__(self, key, value):
         """
@@ -1222,6 +1228,36 @@ class Tags(JavaObject):
             result += str(self.array[i])
         return result
 
+    def find(self, name):
+        """
+        Returns the Tag that matches the name.
+        :param name: the string representation of the tag
+        :type name: str
+        :return: the tag, None if not found
+        :rtype: Tag
+        """
+        result = None
+        for t in self.array:
+            if str(t) == name:
+                result = Tag(t.jobject)
+                break
+        return result
+
+    @classmethod
+    def get_tags(cls, classname, field):
+        """
+        Instantiates the Tag array located in the specified class with the given field name.
+        Example: tag_array("weka.classifiers.functions.SMO", "TAGS_FILTER")
+        :param classname: the classname in which the tags reside
+        :type classname: str
+        :param field: the field name of the Tag array
+        :type field: str
+        :return: the Tags objects
+        :rtype: Tags
+        """
+        classname = "L" + classname.replace(".", "/") + ";"
+        return Tags(jobject=javabridge.get_static_field(classname, field, "[Lweka/core/Tag;"))
+
 
 class SelectedTag(JavaObject):
     """
@@ -1311,6 +1347,66 @@ def split_options(cmdline):
             cmdline))
 
 
+def backquote(s):
+    """
+    Backquotes the string.
+    :param s: the string to process
+    :type s: str
+    :return: the backquoted string
+    :rtype: str
+    """
+    return javabridge.static_call(
+        "Lweka/core/Utils;", "backQuoteChars", "(Ljava/lang/String;)Ljava/lang/String;", s)
+
+
+def backquote(s):
+    """
+    Backquotes the string.
+    :param s: the string to process
+    :type s: str
+    :return: the backquoted string
+    :rtype: str
+    """
+    return javabridge.static_call(
+        "Lweka/core/Utils;", "backQuoteChars", "(Ljava/lang/String;)Ljava/lang/String;", s)
+
+
+def unbackquote(s):
+    """
+    Un-backquotes the string.
+    :param s: the string to process
+    :type s: str
+    :return: the un-backquoted string
+    :rtype: str
+    """
+    return javabridge.static_call(
+        "Lweka/core/Utils;", "unbackQuoteChars", "(Ljava/lang/String;)Ljava/lang/String;", s)
+
+
+def quote(s):
+    """
+    Quotes the string if necessary.
+    :param s: the string to process
+    :type s: str
+    :return: the quoted string
+    :rtype: str
+    """
+    return javabridge.static_call(
+        "Lweka/core/Utils;", "quote", "(Ljava/lang/String;)Ljava/lang/String;", s)
+
+
+def unquote(s):
+    """
+    Un-quotes the string.
+    :param s: the string to process
+    :type s: str
+    :return: the un-quoted string
+    :rtype: str
+    """
+    return javabridge.static_call(
+        "Lweka/core/Utils;", "unquote", "(Ljava/lang/String;)Ljava/lang/String;", s)
+
+
 def to_commandline(optionhandler):
     """
     Generates a commandline string from the OptionHandler instance.
@@ -1343,4 +1439,331 @@ def from_commandline(cmdline, classname=None):
         return handler
     else:
         c = get_class(classname)
-        return c(jobject=handler.jobject)
\ No newline at end of file
+        return c(jobject=handler.jobject)
+
+
+class AbstractParameter(OptionHandler):
+    """
+    Ancestor for all parameter classes used by SetupGenerator and MultiSearch.
+    """
+
+    def __init__(self, classname=None, jobject=None, options=None):
+        """
+        Initializes the specified parameter using either the classname or the supplied JB_Object.
+        :param classname: the classname of the search parameter
+        :type classname: str
+        :param jobject: the JB_Object to use
+        :type jobject: JB_Object
+        :param options: the list of commandline options to set
+        :type options: list
+        """
+        if jobject is None:
+            jobject = AbstractParameter.new_instance(classname)
+        self.enforce_type(jobject, "weka.core.setupgenerator.AbstractParameter")
+        super(AbstractParameter, self).__init__(jobject=jobject, options=options)
+
+    @property
+    def prop(self):
+        """
+        Returns the currently set property to apply the parameter to.
+        :return: the property
+        :rtype: str
+        """
+        return javabridge.call(self.jobject, "getProperty", "()Ljava/lang/String;")
+
+    @prop.setter
+    def prop(self, s):
+        """
+        Sets the property to apply the parameter to.
+        :param s: the property
+        :type s: str
+        """
+        javabridge.call(self.jobject, "setProperty", "(Ljava/lang/String;)V", s)
+
+
+class ListParameter(AbstractParameter):
+    """
+    Parameter using a predefined list of values, used by SetupGenerator and MultiSearch.
+    """
+
+    def __init__(self, jobject=None, options=None):
+        """
+        Initializes the specified parameter using either its classname or the supplied JB_Object.
+        :param jobject: the JB_Object to use
+        :type jobject: JB_Object
+        :param options: the list of commandline options to set
+        :type options: list
+        """
+        classname = "weka.core.setupgenerator.ListParameter"
+        if jobject is None:
+            jobject = ListParameter.new_instance(classname)
+        self.enforce_type(jobject, "weka.core.setupgenerator.ListParameter")
+        super(ListParameter, self).__init__(jobject=jobject, options=options)
+
+    @property
+    def values(self):
+        """
+        Returns the currently set values.
+        :return: the list of values (strings)
+        :rtype: list
+        """
+        return split_options(javabridge.call(self.jobject, "getList", "()Ljava/lang/String;"))
+
+    @values.setter
+    def values(self, l):
+        """
+        Sets the list of values to apply.
+        :param l: the list of values (strings)
+        :type l: list
+        """
+        javabridge.call(self.jobject, "setList", "(Ljava/lang/String;)V", join_options(l))
+
+
+class MathParameter(AbstractParameter):
+    """
+    Parameter using a math expression for generating values, used by SetupGenerator and MultiSearch.
+    """
+
+    def __init__(self, jobject=None, options=None):
+        """
+        Initializes the specified parameter using either its classname or the supplied JB_Object.
+        :param jobject: the JB_Object to use
+        :type jobject: JB_Object
+        :param options: the list of commandline options to set
+        :type options: list
+        """
+        classname = "weka.core.setupgenerator.MathParameter"
+        if jobject is None:
+            jobject = ListParameter.new_instance(classname)
+        self.enforce_type(jobject, "weka.core.setupgenerator.MathParameter")
+        super(MathParameter, self).__init__(jobject=jobject, options=options)
+
+    @property
+    def minimum(self):
+        """
+        Returns the currently set minimum value.
+        :return: the minimum
+        :rtype: float
+        """
+        return javabridge.call(self.jobject, "getMin", "()D")
+
+    @minimum.setter
+    def minimum(self, m):
+        """
+        Sets the new minimum value.
+        :param m: the minimum
+        :type m: float
+        """
+        javabridge.call(self.jobject, "setMin", "(D)V", m)
+
+    @property
+    def maximum(self):
+        """
+        Returns the currently set maximum value.
+        :return: the maximum
+        :rtype: float
+        """
+        return javabridge.call(self.jobject, "getMax", "()D")
+
+    @maximum.setter
+    def maximum(self, m):
+        """
+        Sets the new maximum value.
+        :param m: the maximum
+        :type m: float
+        """
+        javabridge.call(self.jobject, "setMax", "(D)V", m)
+
+    @property
+    def step(self):
+        """
+        Returns the currently set step value.
+        :return: the step
+        :rtype: float
+        """
+        return javabridge.call(self.jobject, "getStep", "()D")
+
+    @step.setter
+    def step(self, s):
+        """
+        Sets the new step value.
+        :param s: the step
+        :type s: float
+        """
+        javabridge.call(self.jobject, "setStep", "(D)V", s)
+
+    @property
+    def base(self):
+        """
+        Returns the currently set base value.
+        :return: the base
+        :rtype: float
+        """
+        return javabridge.call(self.jobject, "getBase", "()D")
+
+    @base.setter
+    def base(self, b):
+        """
+        Sets the new base value.
+        :param b: the base
+        :type b: float
+        """
+        javabridge.call(self.jobject, "setBase", "(D)V", b)
+
+    @property
+    def expression(self):
+        """
+        Returns the currently set expression.
+        :return: the expression
+        :rtype: str
+        """
+        return javabridge.call(self.jobject, "getExpression", "()Ljava/lang/String;")
+
+    @expression.setter
+    def expression(self, e):
+        """
+        Sets the new expression.
+        :param e: the expression
+        :type e: str
+        """
+        javabridge.call(self.jobject, "setExpression", "(Ljava/lang/String;)V", e)
+
+
+class SetupGenerator(OptionHandler):
+    """
+    Allows generation of large number of setups using parameter setups.
+    """
+
+    def __init__(self, jobject=None, options=None):
+        """
+        Initializes the specified classifier using its classname or the supplied JB_Object.
+        :param jobject: the JB_Object to use
+        :type jobject: JB_Object
+        :param options: the list of commandline options to set
+        :type options: list
+        """
+        classname = "weka.core.SetupGenerator"
+        if jobject is None:
+            jobject = OptionHandler.new_instance(classname)
+        else:
+            self.enforce_type(jobject, classname)
+        super(SetupGenerator, self).__init__(jobject=jobject, options=options)
+
+    @property
+    def base_object(self):
+        """
+        Returns the base object to apply the setups to.
+        :return: the base object
+        :rtype: JavaObject
+        """
+        return JavaObject(javabridge.call(self.jobject, "getBaseObject", "()Ljava/io/Serializable;"))
+
+    @base_object.setter
+    def base_object(self, obj):
+        """
+        Sets the base object to apply the setups to.
+        :param obj: the object to use (must be serializable!)
+        :type obj: JavaObject
+        """
+        if not obj.is_serializable:
+            raise Exception("Base object must be serializable: " + obj.classname)
+        javabridge.call(self.jobject, "setBaseObject", "(Ljava/io/Serializable;)V", obj.jobject)
+
+    @property
+    def parameters(self):
+        """
+        Returns the list of currently set search parameters.
+        :return: the list of AbstractSearchParameter objects
+        :rtype: list
+        """
+        array = JavaArray(javabridge.call(self.jobject, "getParameters", "()[Lweka/core/AbstractParameter;"))
+        result = []
+        for item in array:
+            result.append(AbstractParameter(jobject=item.jobject))
+        return result
+
+    @parameters.setter
+    def parameters(self, params):
+        """
+        Sets the list of search parameters to use.
+        :param params: list of AbstractSearchParameter objects
+        :type params: list
+        """
+        array = JavaArray.new_instance("weka.core.AbstractParameter", len(params))
+        for idx, obj in enumerate(params):
+            array[idx] = obj.jobject
+        javabridge.call(self.jobject, "setParameters", "([Lweka/core/AbstractParameter;)V", array)
+
+    def setups(self):
+        """
+        Generates and returns all the setups according to the parameter search space.
+        :return: the list of configured objects (of type JavaObject)
+        :rtype: list
+        """
+        result = []
+        enm = javabridge.get_enumeration_wrapper(javabridge.call(self.jobject, "setups", "()Ljava/util/Enumeration;"))
+        while enm.hasMoreElements:
+            result.append(JavaObject(enm.nextElement))
+        return result
+
+
+def main():
+    """
+    Runs a classifier from the command-line. Calls JVM start/stop automatically.
+    Use -h to see all options.
+    """
+    parser = argparse.ArgumentParser(
+        description='Performs option handling operations from the command-line. Calls JVM start/stop automatically.')
+    parser.add_argument("-j", metavar="classpath", dest="classpath", help="additional classpath, jars/directories")
+    parser.add_argument("-action", metavar="action", dest="action", required=True,
+                        help="The action to perform on the options: join=create single string, "\
+                             "split=create array from quoted option, code=generate code from options")
+    parser.add_argument("option", nargs=argparse.REMAINDER, help="The option(s) to process")
+    parsed = parser.parse_args()
+    jars = []
+    if parsed.classpath is not None:
+        jars = parsed.classpath.split(os.pathsep)
+
+    jvm.start(jars, packages=True)
+
+    try:
+        if parsed.action == "join":
+            output = "cmdline = \"" + backquote(join_options(parsed.option)) + "\""
+        elif parsed.action == "split" and len(parsed.option) == 1:
+            output = "options = [\n"
+            opts = split_options(parsed.option[0])
+            for idx, opt in enumerate(opts):
+                if idx > 0:
+                    output += ",\n"
+                output += "    \"" + backquote(opt) + "\""
+            output += "]"
+        elif parsed.action == "code":
+            options = parsed.option[:]
+            cname = None
+            # classname + options?
+            if options[0].find(".") > -1:
+                cname = options[0]
+                options = options[1:]
+            output = "options = [ \n"
+            for idx, opt in enumerate(options):
+                if idx > 0:
+                    output += ",\n"
+                output += "    \"" + backquote(opt) + "\""
+            output += "]\n"
+            if cname is not None:
+                output += 'handler = OptionHandler(JavaObject.new_instance("' + cname + '"))\n'
+                output += 'handler.options = options\n'
+        else:
+            raise Exception("Unsupported action: " + parsed.action)
+        if output is not None:
+            print(output)
+    except Exception, e:
+        print(e)
+    finally:
+        jvm.stop()
+
+if __name__ == "__main__":
+    try:
+        main()
+    except Exception, ex:
+        print(ex)
